@@ -173,42 +173,52 @@ for(i=0;i<GLOBALS->numfacs;i++)
 	{
 	if(psr)
 		{
-		char *fstr1 = GLOBALS->facs[i-1]->name;
-		char *fstr2 = GLOBALS->facs[i]->name;
-		int p1 = strand_pnt(fstr1);
-		int p2 = strand_pnt(fstr2);
+		int ev1 = GLOBALS->facs[i-1]->n->extvals;
+		int ev2 = GLOBALS->facs[i]->n->extvals;
 
-		if(!root)
+		if(!ev1 && !ev2)
 			{
-			if((p1>=0)&&(p1==p2))
+			char *fstr1 = GLOBALS->facs[i-1]->name;
+			char *fstr2 = GLOBALS->facs[i]->name;
+			int p1 = strand_pnt(fstr1);
+			int p2 = strand_pnt(fstr2);
+	
+			if(!root)
 				{
-				if(!strncmp(fstr1, fstr2, p1))
+				if((p1>=0)&&(p1==p2))
 					{
-					root = GLOBALS->facs[i-1];
-					root->vec_root = root;
-					root->vec_chain = GLOBALS->facs[i];
-					GLOBALS->facs[i]->vec_root = root;
+					if(!strncmp(fstr1, fstr2, p1))
+						{
+						root = GLOBALS->facs[i-1];
+						root->vec_root = root;
+						root->vec_chain = GLOBALS->facs[i];
+						GLOBALS->facs[i]->vec_root = root;
+						}
 					}
 				}
-			}
-			else
-			{
-			if((p1>=0)&&(p1==p2))
+				else
 				{
-				if(!strncmp(fstr1, fstr2, p1))
+				if((p1>=0)&&(p1==p2))
 					{
-					psr->vec_chain = GLOBALS->facs[i];
-					GLOBALS->facs[i]->vec_root = root;
+					if(!strncmp(fstr1, fstr2, p1))
+						{
+						psr->vec_chain = GLOBALS->facs[i];
+						GLOBALS->facs[i]->vec_root = root;
+						}
+						else
+						{
+						root = NULL;
+						}
 					}
 					else
 					{
 					root = NULL;
 					}
 				}
-				else
-				{
-				root = NULL;
-				}
+			}
+			else
+			{
+			root = NULL;
 			}
 		}
 
@@ -440,7 +450,8 @@ build_hierarchy_array (struct ghw_handler *h, union ghw_type *arr, int dim,
 		       const char *pfx, struct tree **res, unsigned int **sig)
 {
   union ghw_type *idx;
-  struct ghw_type_array *base = arr->sa.base;
+  struct ghw_type_array *base =
+    (struct ghw_type_array *) ghw_get_base_type (arr->sa.base);
   char *name = NULL;
 
   if ((unsigned int)dim == base->nbr_dim)
@@ -449,7 +460,7 @@ build_hierarchy_array (struct ghw_handler *h, union ghw_type *arr, int dim,
       sprintf (GLOBALS->asbuf, "%s]", pfx);
       name = strdup_2(GLOBALS->asbuf);
 
-      t = build_hierarchy_type (h, base->el, name, sig);
+      t = build_hierarchy_type (h, arr->sa.el, name, sig);
 
       if (*res != NULL)
 	(*res)->next = t;
